#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

from pyramid.events import subscriber
from z3c.form import button, field
from z3c.form.interfaces import IDataExtractedEvent
from zope.component.interfaces import ISite
from zope.copy import copy
from zope.interface import Interface, Invalid

from pyams_form.form import AJAXAddForm, ajax_config
from pyams_form.schema import CloseButton
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.interfaces import IPortalContext, IPortalTemplate, IPortalTemplateContainer, \
    LOCAL_TEMPLATE_NAME, MANAGE_TEMPLATE_PERMISSION
from pyams_portal.template import PortalTemplate
from pyams_portal.zmi.container import PortalTemplateContainerTable
from pyams_portal.zmi.layout import PortalTemplateLayoutView
from pyams_skin.interfaces import IContentTitle, IPageHeader
from pyams_skin.interfaces.container import ITableElementEditor, ITableElementName
from pyams_skin.interfaces.viewlet import IBreadcrumbItem, IContextActions, \
    IWidgetTitleViewletManager
from pyams_skin.layer import IPyAMSLayer
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.table import DefaultElementEditorAdapter
from pyams_skin.viewlet.breadcrumb import BreadcrumbAdminLayerItem
from pyams_skin.viewlet.menu import MenuItem
from pyams_skin.viewlet.toolbar import ToolbarAction, ToolbarMenuDivider, ToolbarMenuItem
from pyams_utils.adapter import ContextRequestAdapter, adapter_config
from pyams_utils.registry import get_utility, query_utility
from pyams_utils.traversing import get_parent
from pyams_utils.unicode import translate_string
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm, AdminDialogEditForm
from pyams_zmi.interfaces.menu import IPropertiesMenu
from pyams_zmi.layer import IAdminLayer


__docformat__ = 'restructuredtext'

from pyams_portal import _


@adapter_config(context=(IPortalTemplate, IAdminLayer), provides=IBreadcrumbItem)
class PortalTemplateBreadcrumbAdapter(BreadcrumbAdminLayerItem):
    """Portal template breadcrumb adapter"""

    @property
    def label(self):
        name = self.context.name
        if name == LOCAL_TEMPLATE_NAME:
            return _("Presentation template")
        else:
            return name

    @property
    def url(self):
        name = self.context.name
        if name == LOCAL_TEMPLATE_NAME:
            parent = get_parent(self.context, IPortalContext)
            return absolute_url(parent, self.request, 'admin#template-properties.html')
        else:
            return super(PortalTemplateBreadcrumbAdapter, self).url


@adapter_config(context=(IPortalTemplate, IPyAMSLayer), provides=ITableElementName)
class PortalTemplateTableElementNameAdapter(ContextRequestAdapter):
    """Portal template table element name adapter"""

    @property
    def name(self):
        container = get_parent(self.context, IPortalTemplateContainer)
        if container is not None:
            return self.context.name
        context = get_parent(self.context, IPortalContext)
        if context is not None:
            adapter = self.request.registry.queryMultiAdapter((context, self.request),
                                                              ITableElementName)
            if adapter is not None:
                return self.request.localizer.translate(_("{0} (local template)")).format(
                    adapter.name)
        return '--'


@adapter_config(context=(IPortalTemplate, IAdminLayer, Interface), provides=ITableElementEditor)
class PortalTemplateTableElementEditorAdapter(DefaultElementEditorAdapter):
    """Portal template table element editor adapter"""

    view_name = 'admin#layout.html'
    modal_target = False


@adapter_config(context=(IPortalTemplate, IPyAMSLayer, Interface), provides=IPageHeader)
class PortalTemplateHeaderAdapter(DefaultPageHeaderAdapter):
    """Portal template header adapter"""

    icon_class = 'fa fa-fw fa-columns'

    @property
    def title(self):
        translate = self.request.localizer.translate
        # check for templates container
        container = get_parent(self.context, IPortalTemplateContainer)
        if container is not None:
            return translate(_("« {0} »  portal template")).format(self.context.name)
        # check for portal context
        context = get_parent(self.context, IPortalContext)
        if context is not None:
            adapter = self.request.registry.queryMultiAdapter((context, self.request, self.view),
                                                              IContentTitle)
            if adapter is None:
                adapter = IContentTitle(context, None)
            if adapter is not None:
                return adapter.title


#
# Template add views
#

@viewlet_config(name='add-portal-template.action', context=ISite, layer=IAdminLayer,
                view=PortalTemplateContainerTable, manager=IWidgetTitleViewletManager,
                permission=MANAGE_TEMPLATE_PERMISSION, weight=1)
class PortalTemplateAddAction(ToolbarAction):
    """Portal template add action"""

    label = _("Add template")
    url = 'add-portal-template.html'
    modal_target = True


@pagelet_config(name='add-portal-template.html', context=ISite, layer=IPyAMSLayer,
                permission=MANAGE_TEMPLATE_PERMISSION)
@ajax_config(name='add-portal-template.json', context=ISite, layer=IPyAMSLayer, base=AJAXAddForm)
class PortalTemplateAddForm(AdminDialogAddForm):
    """Portal template add form"""

    title = _("Portal templates")
    legend = _("Add shared template")
    icon_css_class = 'fa fa-fw fa-columns'

    fields = field.Fields(IPortalTemplate).select('name')
    edit_permission = MANAGE_TEMPLATE_PERMISSION

    def create(self, data):
        return PortalTemplate()

    def add(self, template):
        context = query_utility(IPortalTemplateContainer)
        context[translate_string(template.name, spaces='-')] = template

    def nextURL(self):
        return absolute_url(self.context, self.request, 'portal-templates.html')


@subscriber(IDataExtractedEvent, form_selector=PortalTemplateAddForm)
def handle_new_template_data_extraction(event):
    """Handle new template form data extraction"""
    container = query_utility(IPortalTemplateContainer)
    name = translate_string(event.data.get('name', ''), spaces='-')
    if name in container:
        event.form.widgets.errors += (Invalid(_("Specified name is already used!")),)


#
# Template properties form
#

@viewlet_config(name='properties.menu',
                context=IPortalTemplate, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=10,
                permission=MANAGE_TEMPLATE_PERMISSION)
class PortalTemplatePropertiesMenu(MenuItem):
    """Portal template properties menu item"""

    def __new__(cls, context, request, view, manager):
        container = get_parent(context, IPortalTemplateContainer)
        if container is None:
            return None
        return MenuItem.__new__(cls)

    label = _("Template properties...")
    label_css_class = 'fa fa-fw fa-edit'

    url = 'properties.html'
    modal_target = True


@pagelet_config(name='properties.html',
                context=IPortalTemplate, layer=IPyAMSLayer,
                permission=MANAGE_TEMPLATE_PERMISSION)
@ajax_config(name='properties.json', context=IPortalTemplate, layer=IPyAMSLayer)
class PortalTemplatePropertiesEditForm(AdminDialogEditForm):
    """Portal template properties edit form"""

    legend = _("Template properties")
    icon_css_class = 'fa fa-fw fa-edit'

    fields = field.Fields(IPortalTemplate).select('name', 'css_class')

    edit_permission = MANAGE_TEMPLATE_PERMISSION

    _renamed = False

    def update_content(self, content, data):
        changes = super().update_content(content, data)
        if changes:
            data = data.get(self, data)
            old_name = content.__name__
            new_name = translate_string(data.get('name'), spaces='-')
            if old_name != new_name:
                parent = content.__parent__
                parent[new_name] = content
                del parent[old_name]
                self._renamed = True
        return changes

    def get_ajax_output(self, changes):
        if self._renamed:
            return {
                'status': 'redirect',
                'location': absolute_url(self.getContent(), self.request, 'admin')
            }
        return super().get_ajax_output(changes)


#
# Template duplication form
#

@viewlet_config(name='duplication.menu', context=IPortalTemplate, layer=IPyAMSLayer,
                view=PortalTemplateLayoutView, manager=IContextActions,
                permission=MANAGE_TEMPLATE_PERMISSION, weight=110)
class PortalTemplateDuplicationMenu(ToolbarMenuItem):
    """Portal template duplication menu item"""

    def __new__(cls, context, request, view, manager):
        container = get_parent(context, IPortalTemplateContainer)
        if container is None:
            return None
        return ToolbarMenuDivider.__new__(cls)

    label = _("Duplicate template...")
    label_css_class = 'fa fa-fw fa-files-o'

    url = 'duplicate.html'
    modal_target = True


class IPortalTemplateDuplicationButtons(Interface):
    """Portal template duplication form buttons"""

    close = CloseButton(name='close', title=_("Cancel"))
    duplicate = button.Button(name='duplicate', title=_("Duplicate this template"))


@pagelet_config(name='duplicate.html', context=IPortalTemplate, layer=IPyAMSLayer,
                permission=MANAGE_TEMPLATE_PERMISSION)
@ajax_config(name='duplicate.json', context=IPortalTemplate, layer=IPyAMSLayer, base=AJAXAddForm)
class PortalTemplateDuplicationForm(AdminDialogAddForm):
    """Portal template duplicate form"""

    legend = _("Duplicate template")
    icon_css_class = 'fa fa-fw fa-files-o'

    fields = field.Fields(IPortalTemplate)
    buttons = button.Buttons(IPortalTemplateDuplicationButtons)

    edit_permission = MANAGE_TEMPLATE_PERMISSION

    def updateWidgets(self, prefix=None):
        super(PortalTemplateDuplicationForm, self).updateWidgets(prefix)
        if 'name' in self.widgets:
            self.widgets['name'].label = _("New template name")

    def updateActions(self):
        super(PortalTemplateDuplicationForm, self).updateActions()
        if 'duplicate' in self.actions:
            self.actions['duplicate'].addClass('btn-primary')

    def create(self, data):
        return copy(self.context)

    def add(self, template):
        container = get_utility(IPortalTemplateContainer)
        container[translate_string(template.name, spaces='-')] = template

    def nextURL(self):
        return absolute_url(self.request.root, self.request, 'admin#portal-templates.html')

    def get_ajax_output(self, changes):
        return {
            'status': 'redirect',
            'location': self.nextURL()
        }


@subscriber(IDataExtractedEvent, form_selector=PortalTemplateDuplicationForm)
def handle_new_template_data_extraction(event):
    """Handle new template form data extraction"""
    container = get_utility(IPortalTemplateContainer)
    name = translate_string(event.data.get('name'), spaces='-')
    if name in container:
        event.form.widgets.errors += (Invalid(_("Specified name is already used!")),)
